#pragma once
#define AP_MAXNAMELEN 32

enum ap_argspec
{
    /* fundamentals */
    AP_SINGLE     = 0x1,
    AP_MULTIPLE   = 0x2,

    AP_MAYBE      = 0x4,
    AP_REQUIRE    = 0x8,

    /* values to use */
    AP_NOARG      = 0x10,
    AP_NEED       = 0x20,

    AP_ARG        = AP_SINGLE | AP_REQUIRE,
    AP_MAYBEARG   = AP_SINGLE | AP_MAYBE,
    AP_MULTIARG   = AP_MULTIPLE,
    AP_ATLEASTONE = AP_REQUIRE,

    AP_BOOL       = 0x100,
    AP_INT        = 0x200,
    AP_STR        = 0x400,

    /* internal */
    AP_NEED_UNSATISFIED = 0x100000
};

struct ap_opt
{
    char        shortname;
    const char  *longname;
    int         argspec;
    void        *val;
    const char  *desc;
    unsigned    maxargs;
};

struct ap_res
{
    const char  *errdef;
    const char  *errmsg;
    char        buf[2 * AP_MAXNAMELEN];
};

int ap_prepare(struct ap_opt *defs, struct ap_res *res);
int ap_parse(struct ap_opt *defs, struct ap_res *res, char **args, unsigned nargs);
void ap_usage(const struct ap_opt *defs, const char *name);
void ap_help(const struct ap_opt *defs, const char *name, const char *prologue, const char *epilogue);

